;TrackLoad
;Mark Spankus 2/2021
;
;Simple track loader for custom program that does not need OS65D
;This program is stored on track 0 in normal OSI boot format starting with 22 00 01 <program data>
;(The load address $2200 is hard-wired into most OSI boot ROMS)
;The program you want to load follows on the next tracks
;with load address hi, load address low, # of pages, <# of pages of data>
;more tracks may follow with the same format
;the loader is located in zero page, so tracks may load at any address including $2200
;The last track specifies execution address address hi, address lo, 0  (3 bytes total)

; $43,$57, BCDRK, $58, $76, $01, #SEC_PAGES, $47,$53

; OSI HARDWARE
;  OSI 505 board / OSI 470 board
;  C000-C003 = PIA (decoded to C000-C00F) (C000/C040/C080/C0C0)
;  C010-C011 = ACIA (Decode to C010-C01F) (C010/C050/C090/C0D0)
;  C020-C03F = NULLREAD (C020/C060/C0A0/C0E0)

;  C000 = PORT A or DDR A
;  C001 = Control Reg A
;  C002 = PORT B or DDR B
;  C003 = Control Reg B
;  C010 = Serial data
;  C011 = Serial Control

; pin assignments
;  -> = output
;  $C000 (DDPIA)
;  PA0 <- !Drive 1 Ready		(0 = drive 1 ready)
;  PA1 <- !TRACK 00				(0 = head on trk 0)
;  PA2 <- !FAULT (nc)
;  PA3 <- !SECTOR (nc)
;  PA4 <- !Drive 2 Ready		(0 = drive 2 ready)
;  PA5 <- !Write Protect		(0 = write protect)
;  PA6 -> Drive Select			(0 = drv 2, 1 = drv 1)
;  PA7 <- !INDEX
;
;  $C002
;  PB0 -> !WRITE ENABLE			(0 = write)
;  PB1 -> !ERASE ENABLE (nc)	(leave 1)
;  PB2 -> !STEP IN				(0 = to trk 39)
;  PB3 -> !RESET STEP 		    (step on 1->0 transition)
;  PB4 -> !FAULT RESET (nc)		(leave 1) (motor on?)
;  PB5 -> Drive Enable			(1 = drive ON) (Or Drive/Side sel. 4 drive system 1=drive 0&1,0=drive 2&3 )
;  PB6 -> !LOW CURRENT (nc)		(leave 1)
;  PB7 -> !HEAD LOAD			(0 = head on disk)
;
;  CA1 <- 20ms clk on 8"
;  CA2
;  CB1 <- 400 mSec clk for RTC operations (2.5x/sec) 5.25",	1000ms for 8"
;  CB2
;

;ROM routine loads 1st track, and jumps to starting address ($2200)
DDPIA = $C000
DDACIA= $C010
LODMOR= $0000


*=$2200		
	LDX #$00
	TXS
INSTAL
	LDA PROGLOC,X
	STA $00,X
	INX
	CPX #<LODEND
	BNE INSTAL
	JMP LODMOR
PROGLOC=*

*=LODMOR  ;this causes a binary relocation error which we ignore because it is desired behavior (packed binary)

;increment track
	LDA DDPIA+2
	AND #$FB	; 1111 1011   DIR=INWARDS TO 39/76
	STA DDPIA+2	; SET 'DIR' PIN
	JSR SDELAY	;short delay
	AND #$F7	; 1111 0111
	STA DDPIA+2	; SET 'STEP' LOW
	JSR SDELAY
	ORA #8		; 0000 1000
	STA DDPIA+2	; SET 'STEP' HIGH
	LDX #$18
	JSR LDELAY   ;longer delay
	LDX #$7F	 ;load head
	STX DDPIA+2
	JSR LDELAY	 ;delay 320 ms
IDXSRT	
	LDA DDPIA
	BMI IDXSRT	 ;wait for index start
	LDA #$03
	STA DDACIA	 ;reset ACIA	
IDXEND
	LDA DDPIA
	BPL IDXEND	 ;wait for index end

	LDA #$58
	STA DDACIA	 ;/1 RTS hi, no irq
HDR1	
	JSR RDISK	; get sector start code
	CMP #$76
	BNE HDR1
	JSR RDISK   ; get sector # 01
	CMP #$01
	BNE HDR1
	JSR RDISK   ;read number of pages in 65D sector
	JSR RDISK   ;read number of pages for bootthis
	STA $FF 	;store # pages to process (-3)
	BEQ GOEND	
	CMP #$0E	;either 0 or invalid sector length causes exit
	BCS GOEND	

	JSR RDISK	;read start addr hi
	STA $FE	 	
	JSR RDISK	;read start addr lo  
	SEC
	SBC #$03	;our header in 65d sector is 3 bytes
	STA $FD
	BCS NODEC
	DEC $FE
NODEC
;	JSR RDISK
;	STA $FF	 	;read num pages

	LDY #$03	;our header in 65d sector is 3 bytes
RLOOP
	JSR RDISK	 ;read data
	BCC RERROR
	STA ($FD),Y
	INY
	BNE RLOOP
	INC $FE
	DEC $FF
	BNE RLOOP
	;LDA #$FF
	;STA DDPIA+2 ;unload head, disable drive
	JMP LODMOR
RERROR
	JMP	($FFFC)	;Exit to RESET vector
LDELAY      
	LDY #$F8	;long delay FF85
	DEY
	BNE LDELAY+2
	EOR $FF,X
	DEX
	BNE LDELAY
SDELAY
	CLC
	RTS
RDISK = *		;Read character from disk, returned in A, carry set=OK clear=error
	LDA DDPIA	;check for index during read
	BPL SDELAY
	LDA DDACIA	;wait for ACIA char
	LSR A
	BCC RDISK
	LDA DDACIA+1
	RTS
GOEND
	LDA #$FF
	STA DDPIA+2 ;unload head, disable drive
	LDX #$02
	LDY #$00
C1JMP
	LDA <GOJMP+2,Y
	STA $012B,Y
	LDA <GOJMP,X 
	STA $00,X	;set C1P Warm Start
	STA $012E,Y
	INY
	DEX
	BPL C1JMP
GOJMP=*
	JMP $0300
	.BYTE $04,$FD ;additional bytes for 65A 'Go'  //0129, Y=0, X=0, A=0, P=4, k=FD (PCh PCl)

LODEND
	.BYTE 0
	

	